from __future__ import unicode_literals

import sys
import warnings

from .core import textile, textile_restricted, Textile
from .version import VERSION

__all__ = ['textile', 'textile_restricted']

__version__ = VERSION
